/* ==================================================== ======== ======= *
 *
 *  uappliImpl.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uappliImpl_hh
#define	_uappliImpl_hh
//pragma ident	"@(#)uappliImpl.hh	ubit:2.11.0"

#include <uappli.hh>

class UInputFlow {
  friend class UAppli;
  friend class UCtrl;

  static int last_id;
  UAppli& appli;
  UDisp& disp;
  const int id;

  // last entered, pressed, armed, browsed object in the Display
  //$$ UEvent *currentEvent, lastEntered, lastPressed;
  //$$ UCtrl  *lastArmed, *dragSource, *dropTarget;
  UEvent lastEnteredEvent, lastPressedEvent;
  UCtrl *lastPressed, *lastArmed, *dragSource, *dropTarget;
  UView *lastEnteredView;
  UBox  *lastEnteredBox;

  // current browsing group (for Menus, Lists, Choices...)
  // (null if no browsing group currently activated)
  UGroup* browse_group;

  // controls the menus and menubars of the application
  class UMenuCtrl& menuCtrl;

  // controls the text selection of the application
  class UTextsel& textsel;

  // cursor being currently shown
  const class UCursor* lastCursor;

  u_card mclickCount;
  u_time mclickTime;
  u_pos  mclickX, mclickY;

  void deleteNotify(class UView* deleted_view);
  void deleteNotify(class UGroup* deleted_group);
  ///< [impl] notifies the application(s) that a view has been deleted

public:
  UInputFlow(class UAppli&, class UDisp&);
  virtual ~UInputFlow();

  int getID() const  {return id;}
  class UDisp&  getDisp()  const {return disp;}
  class UAppli& getAppli() const {return disp.getAppli();}

  class UGroup* getBrowseGroup() const {return browse_group;}
  void setBrowseGroup (UGroup* grp) {browse_group = grp;}
  ///< [impl] gets/sets current browse group.

  class UMenuCtrl& getMenuCtrl() const {return menuCtrl;}
  ///< [impl] returns the Menu Controller.

  class UTextsel& getTextsel() const {return textsel;}
  ///< [impl] returns the text selection of the application.

  void setCursor(class UEvent&, const class UCursor*);

  void winMousePress(UWin *win, UEvent&);
  void winMouseRelease(UWin *win, UEvent&);
  void winMouseMove(UWin *win, UEvent&);
  void winKeyPress(UWin *win, UEvent&);
  void winKeyRelease(UWin *win, UEvent&);
  void winLeave(UWin *win, UEvent&);
  void winDestroy(UWin *win, UEvent&);

  class UBox* findBox(const UEvent& e, u_modes, UView*& found_view);
  //$$void boxEnter(UEvent&, UBox*, UView*, int bstyle);
  void boxEnter(UEvent&, UBox*, int bstyle);
 void boxLeave(UEvent&);

  void boxMousePress(UEvent&);
  void boxMouseRelease(UEvent&);
  void boxMouseMove(UEvent&);
  void boxKeyPress(UEvent&);
  void boxKeyRelease(UEvent&);
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
