/* ==================================================== ======== ======= *
 *
 *  uborder.hh
 *  Ubit Project [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uborder_hh
#define	_uborder_hh
//pragma ident	"@(#)uborder.hh	ubit:2.11.0"
#include <uprop.hh>

/** Border decoration Property
 */
class UBorder : public UProp {
public:
  enum {NONE=0, FLAT, SHADOW, ETCHED};
  ///< predefined decorations.

  static UBorder
    none, empty, flat, shadowIn, shadowOut, etchedIn, etchedOut;
  ///< predefined borders.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UColor &color, &bgcolor;
  /**< internal representations of the foreground and background colors;
   *   These objects can be freely accessed and their content can be modified
   *   by clients. They can't be destroyed by 'delete' (automatical deletion
   *   when the Spinbox is destroyed and they are not pointed by any uref)
   */
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UBorder(int decoration = -ETCHED);
  UBorder(int decoration, UColor& color, UColor& bgcolor,
	  int top_bottom_margin = 1, int left_right_margin = 1, u_modes = 0);
  /**< creates a new Border with custom properties.
   * arguments:
   * - 'decoration' is one of NONE, +/-SHADOW, +/-ETCHED, +/-FLAT (a negative 
   *    value sets an INwards shadow, a positive value an OUTwards shadow)
   * - 'color' and 'bgcolor' are the colors of the shadows. They can be
   *   set to UColor::inherit to inherit colors from their parents (ie. the
   *   boxes that contains them). They can't be destroyed by 'delete' 
   *   (automatical deletion when the UBorder is destroyed and these objects
   *   are not pointed by any uref)
   */

  UBorder(const UBorder&);
  ///< creates a new Border that is a copy of another Border.

  UBorder(bool overlaid, const UArgs&);
  ///< creates an active border (used to create scrollpanes, etc).

  virtual ~UBorder();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void setDecoration(int decoration);
  void setMargin(const UMargin&);

  int getDecoration() const;
  bool isOverlaid() const {return overlaid;}
  const UColor&  getColor() const;
  const UColor&  getBgcolor() const;
  const UMargin& getMargin() const;

  UGroup* getElems() const {return elems;}
  ///< content can be changed but can't be destroyed by 'delete'.

  virtual void update();

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  virtual void putProp(UContext*, class UCtrl*);
  virtual void getSize(const UContext&, UMargin&) const;
  virtual void paint(class UWinGraph&, const UContext&, const URegion &r) const;

protected:
  int     decoration;
  bool    overlaid;
  UMargin margin;
  UGroup* elems;

  virtual void constructs(int decoration);
#endif
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
