/* ==================================================== ======== ======= *
 *
 *  uconf.hh : configuration
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uconf_hh
#define	_uconf_hh
//pragma ident	"@(#)uconf.hh	ubit:2.10.2"

/** configuration of the UAppli.
 */
class UConf {
  
  struct Profile {
    const char* name;
    void (*apply)(UConf&);
    Profile(const char* _name, void (*_apply)(UConf&));
  };

  static std::vector<UConf::Profile> profiles;
  static bool init;

  // IPAQ:	320x400 ???
  // VGA:	640x480
  // SVGA:	800x600
  // XGA:	1024x768
  // (TITANIUM:	1152x768) ???
  // SXGA:	1280x1024
  // UXGA:	1600x1200
  // UXGAW/HDTV 1920x1200
  // QUXGAW:	3840x2400

public:
  enum GraphicsEngine {X11, GL};

  UConf(const char* profile = null);
  virtual ~UConf();

  static bool initDefaultProfiles();   
  static void addProfile(const char* name, void (*apply)(UConf&));
 
  bool setProfile(const char* name);
  void parseOptions(UAppli&, int *argc, char *argv[]);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  const char* locale;

  enum GraphicsEngine graphics_engine;  // can't be changed
  int truecolor_depth;                  // must be specified at init
  /**<
   * - 'truecolor_depth' : the number of bits for representing colors in 
   *    "True Color" mode (default is 24 bits). This is a hint: a smaller value
   *    can actually be used if the display does not support this depth (or if 
   *    it does not support True Colors). A value of 0 means "use server default".
   */

  bool 
    dual_mouse, 
    double_buffering,
    soft_windows,                      // must be specified at init
    iconic,                            // must be specified at init
    sync,                              // must be specified at init
    menu_grab, 
    scrollbar_grab;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  class UStyle* defaultStyle;
  UPix* unknown_image;		///< MUST be defined

  /// scaling factor applied to all windows
  int appli_scale;		///< 0 is the default

  /// maximum size used by the UAppli ( <=0 means : no constraint)
  u_dim max_appli_width, max_appli_height; 

  int scrollpane_mode;
  u_dim scrollpane_bar_size;

  class UWidth  filebox_width;         ///< MUST be defined
  class UHeight filebox_height;        ///< MUST be defined
  int filebox_line_count;              ///< MUST be defined

  u_dim  mouse_click_radius;
  u_time mouse_click_delay;     ///< millisec, for double clicks
  u_time auto_open_menu_delay;  ///< millisec, for opening menus

  class UColor* default_selection_color;    ///< none if pointer is null
  class UColor* default_selection_bgcolor;  ///< none if pointer is null
  class UFont*  default_selection_font;     ///< none if pointer is null

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  static const int COLORMAP_QUANTUM;
  static const int CURSORMAP_QUANTUM;
  static const int FILE_DIR_QUANTUM;
  static const int TABLE_CELL_QUANTUM;
  static const int MAX_CASCADED_MENUS;
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
