/* ==================================================== ======== ======= *
 *
 *  udefs.hh
 *  Ubit Project [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _udefs_hh
#define	_udefs_hh

#include <vector>
#include <string>   // C++ strings

#define _UBIT_VERSION_STRING	"UbiX:2.11.0"
#define _UBIT_VERSION_NO	21100

typedef signed int    u_pos;	// !!must be signed!!
typedef signed int    u_dim;	// !!must be signed!!
typedef signed int    u_card;	// !!must be signed!!
typedef unsigned int  u_count;	// !!must NOT be signed!!
typedef unsigned long u_time;	// !!must NOT be signed!
typedef unsigned long u_modes;	// !!must be coherent with umode.hh!!
typedef signed char   u_state;	// !!must be coherent with umode.hh!!

#define null  0
#define U_MAX(x,y)    (((x) > (y)) ? (x) : (y))
#define U_MIN(x,y)    (((x) < (y)) ? (x) : (y))

/** version of this distrib
 */
extern "C" {
  const char* uversion();
  long uversionNo();
}

/** File status returned by functions that open files.
 */
class UFilestat {
public:
  enum {
    UnknownType   = -5,
    MiscError     = -4,
    NoMemory      = -3,
    InvalidData   = -2,
    CannotOpen    = -1,
    NotLoaded     =  0,
    Loaded        =  1
  };
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Rectangular Region.
 */
struct URegion {
  u_pos x, y;
  u_dim width, height;
 
  URegion() {};
  URegion(const URegion &r) {*this = r;}
  URegion(const URegion *r) {*this = *r;}
  URegion(u_pos rx, u_pos ry, u_dim rwidth, u_dim rheight) {
    x = rx; y = ry; width = rwidth; height = rheight;
  }

  void set(const URegion &r) {*this = r;}
  void set(const URegion *r) {*this = *r;}
  void set(u_pos rx, u_pos ry, u_dim rwidth, u_dim rheight) {
    x = rx; y = ry; width = rwidth; height = rheight;
  }
  
  // effectue l'intersection de this avec clip
  // this est mis a jour (contient l'intersection), clip ne change pas.
  // renvoie:
  // - false si pas d'intersection (region inchangee)
  // - true sinon si region est incluse dans clip (ie. clipping inutile)
  //
  bool setInter(const URegion &clip2);
  bool setInter(const URegion *clip2);
  bool setInter(u_pos x, u_pos y, u_dim w, u_dim h);

  //Note: les regions vides ne sont pas prises en compte!
  void setClosure(const URegion &clip2);
  void setClosure(const URegion *clip2);
  void setClosure(u_pos x, u_pos y, u_dim w, u_dim h);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Box margins.
 */
struct UMargin {
  short top, right, bottom, left;

  UMargin() {}
  UMargin(const UMargin &m) {*this = m;}
  UMargin(short _top_bottom, short _left_right) {
    top  = bottom = _top_bottom; 
    left = right = _left_right;
  }
  UMargin(short _top, short _right, short _bottom, short _left) {
    top = _top; bottom = _bottom; 
    left = _left; right = _right;
  }
  void set(const UMargin &m) {*this = m;}
  void set(short _top_bottom, short _left_right) {
    top = bottom = _top_bottom; 
    left = right = _left_right;
  }
  void set(short _top, short _right, short _bottom, short _left) {
    top  = _top; bottom = _bottom;
    left = _left; right = _right;
  }
  void incr(const UMargin &m) {
    top  += m.top; bottom += m.bottom; 
    left += m.left; right += m.right;
  }
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */
// pre defs.

class UBrick;

class UNumber;
class UIntg;
class UFloat;

class UProp;
class UColor;
class UBgcolor;
class UFont;
class UBorder;
class UCursor;

class UElem;
class UStr;
class UPix;
class UIma;

class UCond;
class UOn;
class UFlag;

class UMode;
class UGroup;
class UBox;
class UWin;
class UArgs;
class UView;

class UDisp;
class UAppli;
class UInput;
class UTimer;

class UEvent;
class UGraph;
class UWinGraph;
class ULink;
class UContext;
class UStyle;

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
