/* ==================================================== ======== ======= *
 *
 *  uedit.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uedit_hh
#define	_uedit_hh
//pragma ident	"@(#)uedit.hh	ubit:2.11.0"

#include <uprop.hh>

/** make a Container object editable.
 * exemple: 
 * <pre>
 *   UStr& str = ustr("abcd");
 *   UBox& box = ubox(uedit() + str);
 * </pre>
 * will make string 'str' editable inside container 'box'
 */
class UEdit : public UProp {
public:
  UEdit();
  virtual ~UEdit();
  ///< constructor and destructor.

  friend UEdit &uedit() {return *(new UEdit());}
  ///< creator shortcut (same as *new UEdit()).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UEdit& enableReturn(bool yes = true);
  ///< the "Return" key can break the line if argument is true

  const UStr *getCaretStr() const;
  const UStr *getCaretStr(int& pos) const;
  /**< returns the UStr that contains the caret and the position of the caret in this string.
   * Notes: 
   * - returns null if the caret is not set
   * - beware that a Box can contain several UStr. The returned position
   *   is relative to the string that contains the caret (possible other
   *   strings are not taken into account) */

  void setCaretStr(UStr*, int pos);
  void setCaretStr(UStr*, int pos, bool update_view);
  /**< sets the UStr that contains the caret and the position of the caret in this string.
   * Notes: 
   * - pos = 0 means beginning of string; pos = -1 end of string
   * - this UStr must be in the same Box parent as the UEdit */

  virtual void update();

  bool nextChar(bool update_views);
  bool previousChar(bool update_views);
  void deletePreviousChar();
  void deleteChar();

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  virtual void putProp(class UContext*, class UCtrl*);

  virtual void addingTo(class ULink *selflink, UGroup *parent);
  virtual void removingFrom(class ULink *selflink, UGroup *parent);
  ///< NOTE that this function require a specific destructor.

  // prototype for non-warped text (UView)
  virtual void paint(class UWinGraph&, class UContext*, const URegion &r) const;

  // prototype for warped text (UFlowView)
  virtual void paint(class UWinGraph&, class UContext*, const URegion &r, 
		     int offset, int cellen) const;

private:
  uref<UStr>  caret_str;  // the UStr that contains the caret (if any)
  uref<UCall> calls;      // callbacks
  int  caret_pos;         // the position of the caret in the caret string
  bool enable_Return;     // true if the "Return" key can break the line

  void callbacks(UEvent&);
  void kpressed(UEvent&);
  void mpressed(UEvent&);
  void mreleased(UEvent&);
#endif
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
