/* ==================================================== ======== ======= *
 *
 *  ufilebox.hh
 *  Ubit Project  [Elc][2002]
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ufilebox_hh
#define	_ufilebox_hh
//pragma ident	"@(#)ufilebox.hh	ubit:2.8.1"


/* File chooser.
 * This object is a subclass of UBox.
 * You can include it into a UDialog to create a File Dialog Choser:
 * - example: UDialog& filedial = udialog(ufilebox())
 */
class UFilebox : public UBox {
public:
  UFilebox(const UArgs& a = UArgs::none);
  ///< constructor.

  friend UFilebox& ufilebox(const UArgs& a = UArgs::none);
  ///< creator shortcut: equivalent to: *new UFilebox().

  virtual ~UFilebox();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  const UStr& getDir() const;
  ///< returns the directory of the file (without the file name).

  const UStr& getName() const;
  ///< returns the name of the file (without the directory).

  const UStr& getPath() const;
  ///< returns the full pathname: getDir()/getName().

  const UStr& getFilter() const;
  ///< returns the extension filter.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void setDir(const UStr&);
  ///< note that setDir and setFilter reload the directory.

  void setName(const UStr&);

  void setFilter(const UStr&);

  virtual void changeDir(UStr* dirpath);
  /**< sets the directory in a relative way.
   * notes:
   * - goes to the parent directory if arg is ".."
   * - goes to subdirectory "xxx" if arg is "xxx" */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void rescan();
  ///< rescans the directory and updates the display.

  virtual void okBehavior(class UEvent&);
  virtual void cancelBehavior(class UEvent&);
  virtual void selectBehavior(class UEvent&);
  /**< these functions implement the behavior of the File box and can be redefined.
   * conditions: these fcts. are called when:
   * - selectBehavior : an item is selected (= simple clicked) 
   * - okBehavior : an item is double clicked or the OK button is clicked
   * - cancelBehavior : the Cancel button is clicked
   *
   * actions:
   * - selected, and cancelBehavior close the window.
   * - then okBehavior() fires the 'UOn::action' callbacks that are 
   *   attached to the FileDialog
   * - and selectBehavior fires the 'UOn::childSelect' callbacks
   * examples:
   *  -  ufilebox( UOn::action / ucall(this, &MyClass::memberfun, arg) )
   *  -  ufilebox( UOn::action / ucall(globfun, arg))
   */

protected:
  UStr fname, fdir, ffilter;
  mutable UStr fdir2, fpath;
  class UScrollpane *scrollpane;
  class UBox *viewport;// la boite contenant les colonnes d'items
  class UButton *show_attributes, *show_dotfiles, *show_dirs_only;
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */

