/* ==================================================== ======== ======= *
 *
 *  umenuImpl.hh
 *  Ubit Project [Elc::2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _umenuImpl_hh
#define	_umenuImpl_hh
//pragma ident	"@(#)umenuImpl.hh	ubit:2.8.1"


class UMenuCtrl {
  friend class UMenu;
  friend class UMenubar;

  class UInputFlow& iflow;
  // stack of menus that are currently being grabbed
  // (the size of this stack is CASCADED_MENU_MAX_COUNT+1)
  UGroup **menu_stack; 
  int    menu_count;
  UMenu  *active_menu;
  UView  *active_opener, *possible_opener, *possible_closer;

  UTimer *open_timer, *close_timer;
  UMenu *timeout_open_menu, *timeout_close_menu;
  UView *timeout_opener, *timeout_closer;

public:
  UMenuCtrl(class UInputFlow*);

  bool isActive() const;
  ///< is a menu active ?

  bool isActive(UGroup *menu) const;
  ///< is 'this' menu active?.

  UMenu* getActiveMenu() const {return active_menu;}
  ///< returns the menu that is active (null otherwise).

  // NB: closeSubMenus calls closeAllMenus if menu is not found
  void closeSubMenus(UMenu* );
  void closeAllMenus(bool stop_auto_open_mode);
  void activateMenu(UView *opener, UMenu*, UGroup* menuGroup);

  // callback functions
  void enterMenuOpener(UEvent&, UMenu*);
  void enterMenuChild(UEvent&, UMenu*);
  void leaveMenuChild(UEvent&);
  void relaxMenuChild(UEvent&);
  void openMenuTimout();
  void closeMenuTimout();
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */

