/* ==================================================== ======== ======= *
 *
 *  umode.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _umode_hh
#define	_umode_hh
//pragma ident	"@(#)umode.hh	ubit:2.9.1"


/** Objects' modes.
 * this class is implementation dependant.
 * only its static fields should be used by clients.
 */
class UMode: public UBrick {
protected:
  friend class UCtrl;
  // NB: bmodes is inherited from UBrick
  u_modes cmodes, bmask, cmask;

public:
  static UMode 
    /// the parent of this brick will be selected (if applicable, see: isSelectable())
    selected,

    /// the parent of this brick will be unselected (if applicable, see: isSelectable())
    unselected,

    /// the parent of this brick will be selectable (see isSelectable())
    canSelect, //canSelectChildren,

    /// the parent of this brick can be armed
    canArm,

    /// the parent of this brick will be highlighted when entered
    enterHighlight, enterHighborder, 

    /// the parent of this brick will be highlighted when activated
    actionHighlight, actionHighborder, //actionWatch,

    /// the parent of this brick can start a drag and drop action
    canDrag, 

    /// the parent of this brick can receive a drag and drop action
    canDrop,

    /// the parent of this brick will ignore events
    ignoreEvents;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  enum BMODES {
    /// this object was created by 'new' can be destroyed by 'delete' 
    CAN_DEL        = 1<<0,

    /// this object is being deleted
    DELETING       = 1<<1,

    /// this object is constant
    UCONST         = 1<<2,

    /// these modes should not be propagated to parents
    DONT_PROPAGATE = CAN_DEL | DELETING | UCONST,   // !dont' forget!

    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    /// update() is NOT invoked when set(), add() or remove() are called
    NO_AUTO_UPDATE = 1<<3,

    /// this object is part of a text selection
    IN_TEXTSEL     = 1<<4,

    /// this object has a vertical orientation (horizontal otherwise)
    IS_VERTICAL	   = 1<<4,

    /// this object has a backwards orientation (forwards otherwise)
    IS_BACKWARDS   = 1<<5,


    /// this object ignores events (its parent will get them)
    IGNORE_EVENTS  = 1<<9,

    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // implicit behaviors

    CAN_BE_AUTO_OPENED       = 1<<10,
    AUTO_OPEN_WIN_CHILD      = 1<<11,
    ///< this object auto opens UWin children that are in CAN_BE_AUTO_OPENED mode

    AUTO_CLOSE_MENU_BEHAVIOR = 1<<12,
    AUTO_CLOSE_MENU_VALUE    = 1<<13,
    ///< if AUTO_CLOSE_MENU_BEHAVIOR true, AUTO_CLOSE_MENU_VALUE defines this behavior (which is the dynamically inherited)

    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // callbacks to which this object is sensitive

    DELETING_CB      = 1<<14,
    CHANGE_CB        = 1<<15,
    CONTAINER_CB     = 1<<16,
    CHILD_CHANGE_CB  = 1<<17,
    CHILD_SELECT_CB  = 1<<18,
    VIEW_PAINT_CB    = 1<<19,
    VIEW_CHANGE_CB   = 1<<20,
    // -- Key: UOn::kpress, UOn::krelease and UOn::ktype
    KEY_CB           = 1<<21,
    // -- Mouse: UOn::mpress, UOn::mrelease
    MOUSE_CB         = 1<<22,
    // -- Mouse: UOn::mclick, UOn::mclick2
    MOUSE_CLICK_CB   = 1<<23,
    // -- Enter, Leave and Cursor: UOn::enter and UOn::leave
    MOUSE_CROSS_CB   = 1<<24,
    // -- Drag & Move: UOn::mdrag and UOn::mmove
    MOUSE_DRAG_CB    = 1<<25,
    MOUSE_MOVE_CB    = 1<<26,  //!no comma after last item!

    // -- UOn::message, UOn::preChildEvent, UOn::anyEvent
    MESSAGE_CB       = 1<<27,
    PRE_CHILD_EVENT_CB = 1<<28,
    ANY_EVENT_CB     = 1<<29,

    CALLBACKS = 
    DELETING_CB|CHANGE_CB|CONTAINER_CB|CHILD_CHANGE_CB|CHILD_SELECT_CB
    |VIEW_PAINT_CB|VIEW_CHANGE_CB
    |KEY_CB|MOUSE_CB|MOUSE_CLICK_CB|MOUSE_CROSS_CB|MOUSE_DRAG_CB|MOUSE_MOVE_CB 
    |MESSAGE_CB|PRE_CHILD_EVENT_CB
    |ANY_EVENT_CB //!no comma after last item!

    //!BEWARE total size must be coherent with typedef u_modes in udefs.hh!
  };

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  enum CMODES {
    //--- Internal MetaTypes
    GROUP             = 1<<0, ///< logical grouping container
    BOX               = 1<<1, ///< graphical viewing container
    FLOATING          = 1<<2, ///< untiled box: random positionning by UPos
    SOFTWIN  	      = 1<<3, ///< simulated "soft" window (NB: softwin => floating)
    HARDWIN           = 1<<4, ///< "hard" movable x window (menus, dialogs)
    INCRUST           = 1<<5, ///< incrusted x windows (see UIncrust)

    FAKE_PARENT       = 1<<6, ///< not taken into account when auto_deleting children
    BORDER_LIST       = 1<<7, ///< list of border frame elements
    SOFTWIN_LIST      = 1<<8, ///< list of softwins controlled by a hardwin

    /// Refer to isShowable() and isShown() methods for details.
    CAN_SHOW          = 1<<11,

    /// paint in double buffering mode
    DOUBLEBUF         = 1<<12,

    /// this object is in the selected mode
    SELECTED          = 1<<13,
    /// this object is modal when shown
    MODAL             = 1<<14,
    //ENABLED (ISTATE::DISABLED used instead)

    // ** This object ...
    /// can be selected (in the same way as a checkbox)
    CAN_SELECT        = 1<<15, 

    /// can select or edit included text
    CAN_SELECT_TEXT   = 1<<16,
    CAN_EDIT_TEXT     = 1<<17,

    /// can be dragged or dropped
    CAN_DRAG          = 1<<18,
    CAN_DROP          = 1<<19,

    /// can be armed (meaning it will be repaint on Uon::arm conditions
    /// and that corresponding object's callbacks will be called)
    CAN_ARM           = 1<<20,
    CAN_ACTION        = 1<<21,

    // ** Higligting:
    // -- on UOn::action conditions
    ACTION_HIGHLIGHT  = 1<<22,
    ACTION_HIGHBORDER = 1<<23,
    //ACTION_WATCH      = 1<<25,  not used!

    // -- on UOn::enter and UOn::leave conditions
    ENTER_HIGHLIGHT   = 1<<24,
    ENTER_HIGHBORDER  = 1<<25,

    //--- Box attributes

    /// this object has a specific renderer (= a UViewStyle object)
    HAS_RENDERER      = 1<<26,
    /// this object has a specific cursor
    HAS_CURSOR        = 1<<27,

    //--- Win attributes

    WIN_MAPPED        = 1<<28 // !no comma after last item!
    //!! BEWARE total size must be coherent with typedef u_modes in udefs.hh !!
  };

  UMode(u_modes bmodes, u_modes cmodes, bool on_off);
  UMode(u_modes bmodes = 0);
  virtual ~UMode() {cleanup();}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void setParentModes(UCtrl *parent) const;

  virtual void addingTo(class ULink *selflink, UGroup *parent);
  virtual void removingFrom(class ULink *selflink, UGroup *parent);
  ///< NOTE that this function require a specific destructor.
};

/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
#endif

