/* ==================================================== ======== ======= *
 *
 *  umsclient.hh : UMS [Ubit Mouse Server] client
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _umsclient_hh
#define	_umsclient_hh
//pragma ident	"@(#)umsclient.hh	ubit:2.11.0"
#include <udefs.hh>
#include <umsproto.hh>

class UMSclient {
  class UInputFlow& iflow;
  class UNatDisp* nap;
  struct hostent* hostinfo;
  const int   remote_port;
  const char* remote_server;
  struct sockaddr_in& rhost;
  int com_sock;

  int openCom(UMS::Ctrl, const char* args, bool keep_cnx_request);
  void receive();

public:
  UMSclient(class UInputFlow&, const char *host, int port);
  ~UMSclient();

  bool openUMS();

  bool sendMessage(int mouse_flow_id, const char* args);
  /**<
   *  mouse_flow == 0 is the standard X mouse.
   *  mouse_flow > 0 are alternate mouses (must be managed by the UMS)
   */

  bool moveMouse(int mouse_flow_id, u_pos x, u_pos y);
  /**<
   *  mouse_flow == 0 is the standard X mouse.
   *  mouse_flow > 0 are alternate mouses (must be managed by the UMS)
   *  x and y are screen coordinates.
   */

  bool pressMouse(int mouse_flow_id, int mouse_button);
  bool releaseMouse(int mouse_flow_id, int mouse_button);
  /**< press and relaese must be balanced 
   * mouse_button is one of UEvent::MButton1, MButton2 etc.
   * NB: only one mouse button at a time ! (should not be ORed !)
   */

  bool sendCommand(int target, UMS::Ctrl, const char* args);
  bool sendCommand(int target, UMS::Ctrl, long arg0, long arg1, long arg2);
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
