/* ==================================================== ======== ======= *
 *
 *  unat.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _unat_hh
#define	_unat_hh
//pragma ident	"@(#)unat.hh	ubit:2.11.0"
# include <X11/Xlib.h>
# include <X11/Xutil.h>
#ifdef GL
# include <GL/gl.h>
# include <GL/glu.h>
# include <GL/glx.h>
#endif
#include <uevent.hh>
#include <ucolor.hh>

//package_private: ====[internal implementation]===========================
//NOTE: this header is PLATFORM DEPENDENT: X-Window version (X11R5/X11R6)

typedef union _XEvent*  UX_Event;        //also in uevent.hh
typedef Display*	UX_Display;
typedef Screen*		UX_Screen;
typedef Visual*		UX_Visual;
typedef XVisualInfo*	UX_VisualInfo;
typedef Colormap	UX_Colormap;
typedef Window		UX_Window;
typedef GC		UX_GC;
typedef XFontStruct*	UX_Font;
typedef Cursor		UX_Cursor;
typedef unsigned long   UX_Color;
typedef Pixmap		UX_Pixmap;
typedef XImage*		UX_Image;
typedef XRectangle	UX_Rectangle;
typedef Atom		UX_Atom;

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */
// sert a catcher les X Errors safely
// usage (att: les { } sont essentiels!)
// {
//    UXtry xtry;
//    xDoTrucQuiPlante();
//    if (!xtry.status()) doOnError();
// }

struct UXtry {
  //NB: pas vraiment thread-safe car xerror doit etre static
  //(mais pas tres grave, le but 1er etant d'eviter des exit intempestifs)
  static int xerror;
  XErrorHandler xhandler;
  static int quietErrorHandler(Display *, XErrorEvent *xer);

  UXtry();
  ~UXtry();
  bool status();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */
//!ATTENTION: les NatGraph sont PARTAGES!
//(en fait il n'y a generalement qu'un seul natgraph pour toute l'appli)

struct UNatGraph {
  UGraph*    graph_lock;
  UX_GC      gc;
  UX_Color   color, bgcolor;
  UX_Font    font;
  short      font_styles, thickness;
  bool       xor_mode;

#ifdef GL
  GLuint     glFontList;      //Active Font
#endif

  UNatGraph(class UNatDisp*);

  void lock(UGraph*);
  void unlock();
  bool isLocked()     {return (graph_lock != null);}

  UX_GC    getGC()      {return gc;}   // !NB: Implementation dependent
  UX_Color getPixel()   {return color;}
  UX_Color getBgpixel() {return bgcolor;}
  UX_Font  getFont()    {return font;}
#ifdef GL
  GLuint getGLFontList() {return glFontList;}
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** native image (encapsulates the corresponding X resources).
 * can have a "transparent background" (i.e. be shaped)
 * (!! mais seul XPM Reader lit images transp !!)
 * NB:
 * - ximashape == null ==> opaque image
 * - ximashape != null ==> transparent background
 */
class UNatIma {
public:
  static const char* const GIF;
  static const char* const JPG;
  static const char* const XPM;
  static const char* const XPM_DATA;

  UNatIma(UNatDisp*, UX_Image xima, UX_Image ximashape);
  /**< constructor.
   * Note:
   * - ximashape == null => opaque image
   * - ximashape != null => transparent background */

  ~UNatIma();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  u_dim getWidth()   const;
  u_dim getHeight()  const;

  const UX_Image getXImage()      const {return xima;}
  const UX_Image getXImageShape() const {return ximashape;}

  bool  isRealized() const;
  bool  isShaped()   const {return ximashape != null;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  static int readImage(UNatDisp*, const char* filename, const char* filetype, 
		       UNatIma*& natima);
  /**< creates a NatIma from an image file.
   * 'filetype' can be null or any of the constants: UNatIma::GIF, JPEG, XPM
   * or XPM_DATA. If 'filetype' is null, the type is deduced from the filename 
   * suffix. This fct returns 'natima' and the UFilestat loading status.
   */

  UNatIma* createScaledCopy(UNatDisp*, float xscale, float yscale);
  ///< creates a scaled copy of this image.

  static UX_Image createScaledCopy(UNatDisp*, UX_Image xima, 
				   float xscale, float yscale);
  ///< returns a new XImage or NULL (if memory exhausted).

  static bool blendImages(UNatDisp*, UX_Image xima1, UX_Image xima2, 
			  float alpha);
  static bool blendImage(UNatDisp*, UX_Image xima1, UX_Color pixel2,
			 float alpha);
  /**< image blending: the resulting XImage is stored in xima1.
   * returns false if images or natdisp are not compatibles
   * note that a trueColor Visual is requested for blending images */

  static UX_Image allocXImage(UNatDisp *nd, u_dim width, u_dim height, 
			     u_dim depth);
  ///< creates and allocate the image data of an XImage in an appropriate way (note that size and depth must be > 0).

  static unsigned long* allocXColors(UNatDisp *nd, 
			      const XColor* colors, int colorCount); 
  ///< allocates the xcolors and returns the conversion table (which must be deleted[] after use).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  /// type of File or Data Reader functions; returns UFilestat and natima (which can be null).
  typedef int (*Reader) (class UNatDisp*, const char *fpath, UNatIma*& natima);

  /// type of File or Data Writer functions; returns UFilestat.
  typedef int (*Writer) (class UNatDisp*, const char *fpath, const UNatIma* natima);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  static int xpmFileReader(UNatDisp*, const char *fpath, UNatIma*& natima);
  static int xpmDataReader(UNatDisp*, const char *xpm_data, UNatIma*& natima);
  static int gifFileReader(UNatDisp*, const char *fpath, UNatIma*& natima);
  static int jpegFileReader(UNatDisp*, const char *fpath,UNatIma*& natima); 
  ///< the avaibility of these readers depends on the compilation and linking options of the Ubit lib

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

protected:
  friend class UGraph;
  friend class UIma;
  friend class UPix;
  friend class UNatPix;

  UNatDisp *natdisp;
  UX_Image xima, ximashape;
  // extensions used by class UIma
  int      lscale;   //LOGICAL scale
  UNatIma  *next; 
#ifdef GL
  GLuint glIma;  // texture Identifiant
  unsigned char *imaData;
  bool makeGLtexture(UNatDisp*);
#endif
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

class UNatPix {
  friend class UGraph;
  friend class UPix;
  UNatDisp *natdisp;
  UX_Pixmap xpix, xpixshape;
  u_dim width, height;
  int depth, lscale;   //lscale = LOGICAL scale (used by UPix)
#ifdef GL
  GLuint glIma;  // texture Identifiant
#endif

public:
  // xpixshape == None ==> opaque image
  // xpixshape != None ==> transparent background
  UNatPix(UNatDisp*, UX_Pixmap xpix, UX_Pixmap xpixshape, 
	  u_dim width, u_dim height, int depth);

  // creates the pixmaps from the images
  // ximashape == null ==> opaque image
  // ximashape != null ==> transparent background
  UNatPix(UNatDisp*, UX_Image xima, UX_Image ximashape);

  UNatPix(UNatDisp*, UNatIma*);
  ~UNatPix();

  void set(UX_Image xima, UX_Image ximashape);

  u_dim getWidth()  const {return width;}
  u_dim getHeight() const {return height;}
  bool isRealized() const {return xpix != None;}
  bool isShaped()   const {return xpixshape != None;}
  const UX_Pixmap getXPixmap()      const {return xpix;}
  const UX_Pixmap getXPixmapShape() const {return xpixshape;}
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

class UNatWin {
  friend class UGraph;
  friend class UWinGraph;
  friend class UNatGraph;
  UX_Window xwin;   // corresponding X Window

public:
  UNatWin();
  ~UNatWin();

  bool isRealized() const {return (xwin != None);}

  void move(class UNatDisp*, u_pos x, u_pos y);
  void resize(class UNatDisp*, u_dim w, u_dim h);
  void show(class UNatDisp*, bool);
  void toBack(class UNatDisp*);
  void toFront(class UNatDisp*);

  /// gets coordinates of the windown in the screen
  void where(class UNatDisp*, u_pos &screen_x, u_pos &screen_y);
  void setTitle(class UNatDisp*, const UStr&);

  /// corresponding X Window
  UX_Window getXWindow() {return xwin;}

  static bool realizeMenu(class UNatDisp*,     UNatWin*, UWin*);
  static bool realizeDialog(class UNatDisp*,   UNatWin*, UWin*);
  static bool realizeFrame(class UNatDisp*,    UNatWin*, UWin*);
  static bool realizeMainFrame(class UNatDisp*,UNatWin*, UWin*);
  static bool realizeIncrust(class UNatDisp*,  UNatWin*, UWin*);
  void reshapeCB(class UNatDisp*, class UView *winview);

  static void manageEvents(class UNatDisp*, UNatWin*, UWin*);

  static void on_message(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_selection(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_any_event(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_misc(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_expose(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_configure(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_mpress(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_mrelease(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_mmove(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_kpress(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_krelease(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_enter(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_leave(class UInputFlow&, UWin*, UView*, UX_Event);
  static void on_focus(class UInputFlow&, UWin*, UView*, UX_Event);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

class UNatDisp {
protected:
  friend class UWinGraph;
  friend class UGraph;
  friend bool UNatWin::realizeMainFrame(UNatDisp*, UNatWin*, UWin *win);

  const int id;
  class UDisp &disp;   // abstract display counterpart
  UX_Display  xdisplay;
  UX_Screen   xscreen;
  UX_Visual   xvisual;
  UX_Colormap xcmap;
  UX_Window   xwin;    // used for getting depth, etc for drawing pixmaps...
  int         xconnection;
  int         depth;
  unsigned long red_mask, green_mask, blue_mask;
  int red_shift, green_shift, blue_shift;

  UX_Font** fontFamilyMap;
  unsigned long fontFamilyCount;

  UX_Cursor* cursormap;
  unsigned long cursormap_count;

  //default Pixmap with same depth as UNatDisp
  UX_Pixmap defaultPixmap;

#ifdef GL
  GLXContext glxc;
  UX_VisualInfo glvisual;      // GL visual info
  GLuint** glFontFamilyMap;   // array of display list ids
#endif
  
  // used by UWinGraph objects for standard toolkit drawing
  UNatGraph* sharedGraph;
  // used by UGraph objects for appli. drawing
  UNatGraph* clientGraph;

  // the UStr where the Server selection must be inserted
  class UStr *server_selection_str;
  int server_selection_pos;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

public:
  struct {
    UX_Atom PRIMARY_SELECTION, SECONDARY_SELECTION, 
      WM_PROTOCOLS, WM_DELETE_WINDOW, WM_TAKE_FOCUS,
      UBIT_SELECTION, UBIT_MESSAGE;
  } atoms;

  UNatDisp(UDisp&);
  virtual ~UNatDisp();

  virtual void realize();
  ///< initializes actual X data.

  virtual bool isRealized() const;
  ///< is X data initialized ?.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  int           getID()          const {return id;}
  UDisp&        getDisp()        const {return disp;}
  ///< returns the corresponding abstract Display counterpart.

  UX_Display    getXDisplay()    const {return xdisplay;}
  UX_Screen     getXScreen()     const {return xscreen;}
  UX_Visual     getXVisual()     const {return xvisual;}
  UX_Colormap   getXColormap()   const {return xcmap;}
  UX_Window     getXRootWindow() const;
  UX_Window     getXWindow()     const {return xwin;}

  unsigned long getRedMask()   const {return red_mask;}
  unsigned long getGreenMask() const {return green_mask;}
  unsigned long getBlueMask()  const {return blue_mask;}
  int getRedShift()            const {return red_shift;}
  int getGreenShift()          const {return green_shift;}
  int getBlueShift()           const {return blue_shift;}
  // !attention: les masks varient suivant l'architecture
  // (big endian ou little endian)

  int getScreenNumber() const;
  ///< number of the Screen used by this UNatDisp.

  int getScreenCount() const;
  ///< total number of Screens available on this X Server.

  int getDepth() const {return depth;}
  ///< depth of this UNatDisp.

  int getScreenDefaultDepth() const;
  ///< default depth of the Screen.

  int getScreenWidth()  const;
  ///< width of the Screen.

  int getScreenHeight() const;
  ///< height of the Screen.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // internals

  class UNatGraph* getSharedGraph() const {return sharedGraph;}
  ///< used by UWinGraph objects for standard toolkit drawing

  class UNatGraph* getClientGraph() const {return clientGraph;}
  ///< used by UGraph objects for client drawing

  UX_Pixmap getDefaultPixmap() const {return defaultPixmap;}
  ///< default Pixmap with same depth as UNatDisp (for XCreateWindow).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // UMS [Ubit Mouse Server]

  void showNotify(UX_Window, bool shows);
  ///< notifies the UMS [Ubit Mouse Server].

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UX_Color getColor(const UColor& c) {return getColor(c.getImpl());}
  UX_Color getColor(class UColorImpl&);
  /**< returns the native color corresponding to this UColor.
   * Note: the color is implicitely realized if necessary */

  bool realizeColor(const UColor& c) {return realizeColor(c.getImpl());}
  bool realizeColor(class UColorImpl&);
  /**< allocates physical resources for this UColor on this Display.
   * Returns true if the color could be allocated.
   * returns false and sets a default color otherwise */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UX_Cursor getCursor(const UCursor&);
  /**< returns the native cursor corresponding to this UCursor.
   * Note: the cursor is implicitely realized if necessary */

  bool realizeCursor(const UCursor&);
  /**< Allocates physical resources for this UCursor on this UDisp.
   * returns true if the cursor was found and false otherwise */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UX_Font getFont(const class UFontDesc&);
  /**< returns the native font corresponding to this UFont.
   * Note: the font is implicitely realized if necessary */
  
  bool realizeFont(const class UFont&);
  bool realizeFont(const class UFontDesc&);
  bool realizeFontFamily(const class UFontFamily&);
  UX_Font loadNatFont(const class UFontFamily&, int styles, int lsize);
  /**< Allocates physical resources for this UFont/UFontDesc on this UDisp.
   * returns true if the font was found and false otherwise */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  int matchVisualProps(int visual_class, int depth_hint);
  /**<
   * searchs if Screen 'screen_no' supports this depth with this visual class.
   * - 'visual_class' = PseudoColor, TrueColor, etc.
   * - the default screen # is 0
   * - tries to to find the closest depth that is compatible with this 
   *   visual class if 'depth_hint' is not supported and it returns it
   * - returns 0 if this visual class is not supported by this screen */

  int matchVisualProps(UX_VisualInfo, int visual_class, int depth_hint);
  ///< also fills the VisualInfo struct that is given as an argument.

  bool setScreen(int screen_no);
  /**<
   * changes screen according to number (default is screen 0).
   * returns false if screen number is invalid.
   * Note: must be called before realize() or eventLoop() */
    
  int setVisualProps(int visual_class, int depth_hint, UX_Colormap);
  /**<
   * sets the requested visual props if supported
   * - this fct. calls matchVisualProps() first and it works in the same way
   * - the Colormap arg. is optional and is automatically created if = None
   * - must be called before realizing X windows 
   *   (ie. before appli->add(), appli->realize() and eventLoop()) */

  int setVisualProps(UX_VisualInfo, UX_Colormap = None);
  ///< sets the requested visual props according to the VisualInfo struct.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void dispatchEvents(UInputFlow&, UX_Event);
  ///< dispatches X Events to Windows (function called by the event loop).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  bool askSelectionOwnership(class UEvent&);
  /**< [impl] the display becomes the owner of the selection of the X Server.
   * the UEvent must be a button event
   */

  void changeSelectionContentRequest(UX_Event, class UTextsel&);
  // Cas ou une autre application demande a obtenir la valeur de la 
  // selection courante (detenue par 'this' appli)
  // l'event doit etre un XSelectionRequestEvent

  void askSelectionContent(class UEvent&, 
			   class UStr* put_to_this_str, int put_to_this_pos);
  /**< [impl] asks the selection of the X Server
   * the UEvent must be a button event. 
   * The server will send a SelectionNotify event to the UNatDisp that will
   * put the result in this UStr at this position
   */

  void retrieveSelectionContentRequest(UX_Event);
  // the XEvent must be a XSelectionEvent event. 
  // recupere le contenu de la selection lorsque le NatDisp recoit un
  // SelectionNotify event apres avoir demande le contenu de la selection 
  // en appellant askSelectionContent()
  // the XEvent must be a XSelectionEvent event. 

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#ifdef GL
  /// initializes GL
  bool realizeGL();

  /// returns the GL Visual Info
  UX_VisualInfo getGLVisual() const {return glvisual;}

  /// sets a GL color from an X Pixel
  void setGLcolor(UX_Color);

  /// gets a GL color from an X Pixel
  void getGLcolor(UX_Color, GLubyte& red, GLubyte& green, GLubyte& blue);

  /// same as getFont but returns a GL display list id
  GLuint getGLXFont(const class UFontDesc*); 
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

class UNatAppli : public UNatDisp {
  friend class UNatDisp;
  UAppli& appli;
public:
  UNatAppli(class UAppli&);
  virtual ~UNatAppli();

  void eventLoop(int& loop_status);  // starts the mainloop or a subloop

  UAppli& getAppli() {return appli;}

  static void getTime(struct timeval&);
  static u_time convertToMilliSec(struct timeval&);
  bool resetTimers(UGroup* timers, struct timeval& delay);
  void fireTimers(UGroup* timers);

  void resetInputs(UGroup* inputs, fd_set& read_set, int& maxfd);
  void cleanInputs(UGroup* inputs);
  void fireInputs(UGroup* inputs, fd_set& read_set);
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
