/* ==================================================== ======== ======= *
 *
 *  uintg.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uintg_hh
#define	_uintg_hh
//pragma ident	"@(#)unumber.hh	ubit:2.11.0"


/** Active Number Interface.
 */
//class UNumber: public UBrick {
class UNumber {
public:
  UNumber();

  virtual void set(const UIntg&)  = 0;
  virtual void set(const UFloat&) = 0;
  virtual void set(const UStr&)   = 0;

  virtual bool equals(const UIntg&)  const = 0;
  virtual bool equals(const UFloat&) const = 0;

  virtual int   getInt()   const = 0;
  virtual float getFloat() const = 0;

  virtual void onChange(UCall&);
  ///< adds a callback that is activated when the objects' value changes.

protected:
  virtual void fire(class UEvent&, const class UOn&) const;
  ///< [impl] calls callback functions.

  virtual void changed(bool update_now);
  ///< [impl] called when object's content is changed.

#ifndef NO_DOC // - - - - - - - - - - - - - - - - - - - - - - - - - - - -
public:
  void addRef() {refcount++;}
  void removeRef();
  u_count getRefCount() const {return refcount;}

private:
  class UChain* cache;  // callbacks
  u_count refcount;
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Active Integer
 */
class UIntg: public UNumber {
  int value;

public:
  UIntg(int val = 0)      : value(val) {}
  UIntg(const UIntg& val) : value(val.value) {}

  friend UIntg& uintg(int val) {return *(new UIntg(val));}
  ///< creator shortcut (same as *new UIng()).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void set(int);
  virtual void set(const UIntg&);
  UIntg& operator=(int val)           {set(val); return *this;}
  UIntg& operator=(const UIntg& val)  {set(val); return *this;}
  ///< sets value.

  virtual void set(const UFloat&);
  ///< converts and sets value.

  virtual void set(const UStr&);
  /**< converts and sets value.
   * arg is supposed to contain a string that represents a valid number */

  virtual void incr(int);
  virtual void incr(const UIntg&);

  virtual void decr(int);
  virtual void decr(const UIntg&);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual bool equals(int) const;
  virtual bool equals(const UIntg&) const;
  virtual bool equals(const UFloat&) const;

  virtual int   getInt()   const {return value;}
  virtual float getFloat() const {return float(value);}
  int get() const {return value;}
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Active Float
 */
class UFloat: public UNumber {
  float value;

public:
  UFloat(float val = 0)     : value(val) {}
  UFloat(const UFloat& val) : value(val.value) {}

  friend UFloat& ufloat(float val) {return *(new UFloat(val));}
  ///< creator shortcut (same as *new UFloat()).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void set(float);
  virtual void set(const UFloat&);
  UFloat& operator=(float val)          {set(val); return *this;}
  UFloat& operator=(const UFloat& val)  {set(val); return *this;}
  /**< sets value.*/

  virtual void set(const UIntg&);
  /**< converts and sets value.*/

  virtual void set(const UStr&);
  /**< converts and sets value.
   * arg is supposed to contain a string that represents a valid number */

  virtual void incr(const UFloat&);
  virtual void decr(const UFloat&);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual bool equals(const UIntg&) const;
  virtual bool equals(const UFloat&) const;
  virtual bool equals(float) const;

  virtual int   getInt()   const {return int(value);}
  virtual float getFloat() const {return value;}
  float get() const {return value;}
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
