/* ==================================================== ======== ======= *
 *
 *  upix.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (Subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _upix_hh
#define	_upix_hh
//pragma ident	"@(#)upix.hh	ubit:2.8.1"
#include <uima.hh>

/** Pixmap Image object.
 * Works in the same way as UIma: see this class.
 * <br>Note: background transparency (ie. image shapes) is supported
 * for XPM images
 */
class UPix: public UIma {
  mutable class UNatPix *natpix;

public:
  static UPix 
    folder, ofolder, doc, diskette, plus, minus, ellipsis, rball, 
    select, cross, check, zoom, left, right, up, down, turn,
    question, stop, ray, clock, bomb, windows, manual, edit, colors;    
  /**<
   * Predefined Pixmap Images (small icons).
   * most of thse pixmaps were originally designed by Ludvig A. Norin
   * for the FVWM window manager. Please refer to file ufvwmpix.hh
   * for notice (more pixmaps are also available in this file)
   *
   * Note some of these pixmaps are shaped (= they have a "transparent"
   * background) */

  UPix(const UStr& filename);
  UPix(const char* filename = null);
  UPix(const char** xpm_data, u_modes = 0);
  //UPix(const char** xpm_data, const UBgcolor&, u_modes = 0);
  /**< 
   * initializes a Pixmap Image object (that will be loaded at a later time).
   * see UIma::UIma for details
   * Notes: 
   * - background transparency (ie. image shapes) is supported for XPM images
   * - 'xpm_data' is <b>not</b> copied internally and must <b>not</b> be freed
   */

  friend UPix& upix(const UStr& filename)  {return *new UPix(filename);}
  friend UPix& upix(const char* filename)  {return *new UPix(filename);}
  friend UPix& upix(const char** xpm_data) {return *new UPix(xpm_data);}
  ///< creator shortcuts (see constructor for details)

  virtual ~UPix();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  class UNatPix* getNatPix() const {return natpix;}
  //< [impl]

#ifndef NO_DOC
  friend class UBox;
  friend class UNatPix;

  virtual void cleanCache();
  virtual void getSize(UContext*, u_dim *w, u_dim *h) const;
  virtual void paint(UWinGraph&, UContext*, const URegion &r) const;
#endif
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
