/* ==================================================== ======== ======= *
 *
 *  uscroll.hh
 *  Ubit Project [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uscroll_hh
#define	_uscroll_hh
//pragma ident "@(#)uscroll.hh	ubit:2.11.0"
#include <uview.hh>
#include <ugadgets.hh>


/** Scrollbar gadget.
 *  UOn::change callbacks are activated when the scrollbar value is changed.
 */
class UScrollbar: public UBox {
public:
  static class UScrollbarStyle* style;

  enum SMODES {
    STANDARD = 1, TRANSP = 1<<1, WITH_RAIL = 1<<2 , WITH_FEEDBACK = 1<<3,
    COMPACT_EVENTS = 1<<5
  };

  UScrollbar(int smodes, const UOrient&, const UArgs& a = UArgs::none);
  UScrollbar(const UOrient&, const UArgs& a = UArgs::none);
  /**< 
   * constructor: see also shortcut creator uscrollbar() and UGroup::~UGroup().
   * - 'orient' can be UOrient::horizontal or UOrient::vertical.
   * - 'smodes' can be UScrollbar::DEFAULT or a combination of UScrollbar::SMODEs
  */

  ~UScrollbar();

  friend UScrollbar& uscrollbar(const UOrient&, const UArgs& a = UArgs::none);
  ///< shortcut creator: see constructor.

  friend UScrollbar& uhscrollbar(const UArgs& a = UArgs::none);
  ///< shortcut creator: creates a new horizontal scrollbar

  friend UScrollbar& uvscrollbar(const UArgs& a = UArgs::none);
  ///< shortcut creator: creates a new vertical scrollbar

  virtual const UStyle& getStyle(const UBox* box_parent) const;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  float getValue() const;
  void  setValue(float percent);
  /**< gets/sets scrollbar value (float between 0 and 100).
   * Note: UOn::change callbacks are activated when the value is changed.
   */

  float getScrollerIncrement() const;
  void  setScrollerIncrement(float percent);
  ///< get/sets value increment when clicking on a Scroller (float between 0 and 100).

  float getBackgroundIncrement() const;
  void  setBackgroundIncrement(float percent);
  ///< get/sets value increment when clicking on the background of the scrollbar (float between 0 and 100).

  bool isHorizontal() const;
  bool isVertical() const;
  bool isTransparent() const;

  void setPane(class UPane*);
  ///< attaches the scrollbar to a UPane or a UScrollpane

  void setValueImpl(float percent, bool upd);
   ///< [impl] does not update the associated pane.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // imbricated gadgets

protected:
  class LessScroller: public UButton {
  public:
    static class UStyle* style;
    LessScroller(const UArgs&);
    virtual const UStyle& getStyle(const UBox* box_parent) const;
  };

  class MoreScroller: public UButton {
  public:
    static class UStyle* style;
    MoreScroller(const UArgs&);
    virtual const UStyle& getStyle(const UBox* box_parent) const;
  };

  virtual UBox& createSlider(const UArgs& = UArgs::none);
  virtual UBox& createLessScroller(const UArgs& = UArgs::none);
  virtual UBox& createMoreScroller(const UArgs& = UArgs::none);

#ifndef NO_DOC

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  friend class UScrollbarView;

  uref<UBox> pless_btn, pmore_btn, pslider_btn, pslider_box;
  uref<class UPane> ppane;	// the pane controlled by this scrollbar
  UOrient orient;		// orientation
  UBgcolor bgcolor;		// dynamic bgcolor
  UPos slider_pos;		// the position of the slider
  float value;			// value (percentage)
  float scroller_increment, backgd_increment; // scroller and background increments
  u_pos delta_mouse;
  bool  pressed, entered;
  bool  transparent, with_rail, enter_feedback, drag_feedback;

  virtual void constructs(int smodes);

  virtual void pressSlider(UEvent&);
  virtual void releaseSlider(UEvent&);
  virtual void dragSlider(UEvent&);
  virtual void enterSlider(UEvent&);
  virtual void leaveSlider(UEvent&);
  virtual void pressScroller(UEvent&, int dir);

  virtual void pressSliderBox(UEvent&);
  virtual void releaseSliderBox(UEvent&);
  virtual void dragSliderBox(UEvent&);
  virtual void enterSliderBox(UEvent&);
  virtual void leaveSliderBox(UEvent&);

  virtual float getPercent(UEvent&, UView *slider_btn_view, 
			   UView *slider_box_view, u_dim delta_mouse);
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Pane gadget (for making Viewports).
 *  superclass of UScrollpane.
 */
class UPane: public UBox {
public:
  static UStyle *style;

  UPane(const UArgs& a = UArgs::none);
  ///< constructor: see shortcut creator upane() and UGroup::~UGroup()

  friend UPane& upane(const UArgs& a = UArgs::none);
  ///< shortcut creator: creates a new UPane

  virtual ~UPane();

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  float getXScroll();
  float getYScroll();
  ///< returns the current scroll values (float values between 0 and 100).

  void setXScroll(float _xscroll);
  void setYScroll(float _yscroll);
  void setScroll(float xscroll, float yscroll);
  ///< scrolls the pane (and the associated scrollbars when applicable) float values are between 0 and 100.

  void scrollImpl(float new_xscroll, float new_yscroll, 
		  bool upd_x, bool upd_y, bool update_pane);
  ///< [impl] does not update the associated scrollbars.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UBox *getViewport();
  /**< returns the current Viewport (if any; null otherwise).
   * The <b>viewport</b> is the scrolled box in the center area.
   */

  UView *getViewportView(UView *pane_view);
  /**< returns the corresponding Viewport View.
   * Returns the UView of the current Viewport that is contained
   * in Pen view that is given as an argument
   */

  UScrollbar* getVScrollbar();
  UScrollbar* getHScrollbar();
  ///< return the Vertical and Horizontal Scrollbars (if any; null otherwise)

  //void setVScrollbar(UScrollbar*, bool add_to_pane = true);
  //void setHScrollbar(UScrollbar*, bool add_to_pane = true);
  /**<
   *  sets the Vertical and Horizontal Scrollbars.
   *  If 'add_to_pane' is true (default case), the scrollbar is added to
   *  the Pane child list and it will appear inside the Pane.
   *  (otherwise, the scrollbar is NOT added to the Pane and MUST be added
   *  to parent somewhere else)
   *  This makes it possible to CONTROL a Pane by a scrollbar 
   *  that is not included in this Pane. */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

#ifndef NO_DOC
  friend class UPaneView;

  UBoxLink* getViewportLink();
  void setVScrollbarImpl(class UScrollbar* vs) {vscrollbar = vs;}
  void setHScrollbarImpl(class UScrollbar* hs) {hscrollbar = hs;}

protected:
  UScrollbar *hscrollbar, *vscrollbar;
  float xscroll, yscroll;

  virtual void viewResized(UEvent&);
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** UScrollpane = UPane with scrollbars
 * see also: UPane
 */
class UScrollpane: public UPane {
public:
  UScrollpane(const UArgs& a = UArgs::none);
  /**< constructor: creates a pane with 2 scrollbars.
   * See also: UPane and shortcut creator uscrollpane() */

  UScrollpane(int v_scrollbar_mode, int h_scrollbar_mode, 
	      const UArgs& a = UArgs::none);
  /**< constructor: creates a pane with 0, 1 or 2 scrollbars.
   * the vertical (resp. horizontal) scrollbar is not created if
   * v_scrollbar_mode (resp. h_scrollbar_mode) is false.
   * See also: UPane and shortcut creator uscrollpane() */

  friend UScrollpane& uscrollpane(const UArgs& a = UArgs::none);
  ///< shortcut creator: creates a pane with 2 scrollbars

  friend UScrollpane& uscrollpane(int Vscrollbar_policy, int Hscrollbar_policy,
				  const UArgs& a = UArgs::none);
  ///< shortcut creator: creates a pane with 0, 1 or 2 scrollbars.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

protected:
  void constructs(int vs_mode, int hs_mode, const UArgs&);
};


/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Scrollbar View Renderer.
 *  used by: UScrollbar
 *  - you don't need to use this class except in very specific cases
 */
class UScrollbarView : public UView {
public: 
  static  UViewStyle style;  // renderer
  virtual UViewStyle* getViewStyle() {return &style;}

  UScrollbarView(UBoxLink*, UView* parview, UWinGraph*);

  virtual void setScroll(float percent);
  //int getScrollMin() {return scrollmin;}
  //int getScrollMax() {return scrollmax;}

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  friend class UScrollbar;
  static UView* makeView(UBoxLink*, UView* parview, UWinGraph*);
  virtual bool doLayout(UContext&, class UViewLayout&);
  virtual void doUpdate(UContext&, URegion r, URegion clip, class UViewUpdate&);

private:
  //int scrollmin, scrollmax;
#endif
};


/** Pane View Renderer.
 *  used by: UPane
 *  - you don't need to use this class except in very specific cases
 */
class UPaneView: public UView {
public:
  static  UViewStyle style;  // renderer
  virtual UViewStyle* getViewStyle() {return &style;}

  UPaneView(UBoxLink*, UView* parview, UWinGraph*);
  virtual ~UPaneView() {}

  static UView* makeView(UBoxLink*, UView* parview, UWinGraph*);
  ///< makeView() is a "static creator" used by UViewStyle to make a new view.

  UPane *getPane(); 
  ///< same as getBox() with an UPane cast.

  u_dim getXScroll() {return xscroll;}
  u_dim getYScroll() {return yscroll;}

  virtual void setXScroll(u_dim scr) {xscroll = scr;}
  virtual void setYScroll(u_dim scr) {yscroll = scr;}
  ///< note that using these functions is not a good idea if this pane is already controlled by scrollbars.

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  friend class UPane;
  //friend class UScrollbar;
  UMargin margin;
  u_dim xscroll, yscroll;
#endif
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */

